package {
	import flash.display.*;
	import flash.geom.*;
	import flash.events.*;
	import Handle;
	
	/**
	 * @author nicoptere
	 * http://en.nicoptere.net/
	 */
	public class Main extends MovieClip 
	{
		private var steps:int = 100;
		
		private var i:Number;
		private var j:Number;
		
		private var pathes:Array;
		private var curves:Array;
		
		private var path0:Path;	
		private var path1:Path;		
		
		private var bd:BitmapData;

		public function Main() 
		{
			//quality
			/**********************************************************/
			
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			stage.quality = StageQuality.MEDIUM;
			
			//background pattern setup
			/**********************************************************/
			
			bd =new BitmapData( 2, 2, true, 0x00000000 );
			bd.setPixel32( 0,0, 0xFF000000 );
			bd.setPixel32( 1,1, 0xFF000000 );
			
			// pathes setup
			/**********************************************************/
			
			var handlesNumber:int = 8;
			
			path0 = new Path( handlesNumber, stage.stageWidth, stage.stageHeight / 2 );
			addChild( path0 );
			
			path1 = new Path( handlesNumber );
			addChild( path1 );
			
			j = path0.numChildren;
			while( j-- )
			{
				var p0:Handle = path0.getChildAt( j ) as Handle;
				var p1:Handle = path1.getChildAt( j ) as Handle;
				
				p1.x = p0.x + Math.random() - .5 * 25;
				p1.y = stage.stageHeight/2 + Math.random() * stage.stageHeight / 2;
				
			}
			
			pathes = [ path0, path1 ];
			
			// listeners
			/**********************************************************/
			
			addEventListener( Event.ENTER_FRAME, update );
			
		}
		
		/**
		 * retrieves the handles contained in each pathes instances
		 * @param	path: a ppath object
		 * @return  the list of handles
		 */
		private function getHandles( path:MovieClip ):Array
		{
			
			var handles:Array = [];
			var j:int = path.numChildren;
			while( j-- )
			{
				handles.push( path.getChildAt( j ) );
			}
			return handles;
			
		}
		/**
		 * recursively finds the interpolated pathes between the 2 sets of points (Handles are points basically)
		 * @param	pathes: begin and end pathes  
		 */
		private function processCurves( pathes:Array ):void
		{
			curves = [];
			
			//you can try a .01+Math.random();
			var smoothness:Number = .05;
			
			
			var i:int = steps;
			var c:Curve;
			
			var tmp:Array;
			var p:Point;
			
			var p0:MovieClip;	
			var p1:MovieClip;	
			
			var dx:Number;
			var dy:Number;
			var L:int = pathes[ 0 ].numChildren;
			while( i-- )
			{
				
				tmp = [];
				j = L;
				while( j-- )
				{
					//actually we could loop through X pathes, 
					//as we only have two, containing the same number of handles it is simpler that way
					p0 = pathes[ 0 ].getChildAt( j );
					p1 = pathes[ 1 ].getChildAt( j );
					
					dx = ( ( p1.x - p0.x ) / steps ) * i;
					dy = ( ( p1.y - p0.y ) / steps ) * i;
					
					//intrpolating between the 2 pathes handles
					p = new Point( p0.x + dx , p0.y + dy );
					tmp.push( p );
					
				}
				
				//creates and Stores a smoothed curve
				c = new Curve( Spline.QuadraticPath( tmp, smoothness ) );
				curves.push( c );
				
			}
		}
		
		/**
		 * update loop
		 * @param	e
		 */
		private function update( e:Event = null ) :void
		{
			//draws the background
			this.graphics.clear();
			graphics.beginBitmapFill( bd );
			graphics.drawRect( 0, 0, stage.stageWidth, stage.stageHeight );
			graphics.endFill();
			graphics.lineStyle( .1, 0x000000, 1 );
			
			//processes and draws the curves
			processCurves( pathes );
			var c:Curve;
			for( i = 0; i < curves.length; i++ )
			{
				c = curves[ i ];
				c.render( this.graphics );
			}
		}
	}
}
